/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


/**
 * \file CubeFileHybridLayout.h
 * \brief It provides names for the files in the CUBE report hybrid layout, with anchor and directory.
 */
#ifndef __FILE_HYBRID_LAYOUT_H
#define __FILE_HYBRID_LAYOUT_H


#include <inttypes.h>
#include <stdint.h>
#include <map>
#include <string>
#include <vector>
#include <iostream>

#include "CubeMetric.h"

#include "CubeFileBaseLayout.h"

namespace cube
{
class FileHybridLayout : public FileBaseLayout
{
protected:

    virtual std::string
    getPathToAnchor();
    virtual std::string
    getPathToData();
    virtual std::string
    getPathToMetricData( cube::Metric* );
    virtual std::string
    getPathToMetricIndex( cube::Metric* );

    virtual std::string
    getAnchorName();
    virtual std::string
    getMetricDataName( cube::Metric* );
    virtual std::string
    getMetricIndexName( cube::Metric* );


public:
    FileHybridLayout( const std::string& name ) : FileBaseLayout( name )
    {
    };

    static std::string
    getDataDirExtension()
    {
        return FileBaseLayout::getAnchorExtension() + ".data";
    };
};
};

#endif
