/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
/**
 *
 * \file CubeUnsignedValue.h
 * \brief Provides the "unsigned int" (32bit) build-in type as "Value"
 ************************************************/
#ifndef __UNSIGNED_VALUE_H
#define __UNSIGNED_VALUE_H

#include <istream>
#include "CubeValue.h"

#include <iostream>
using namespace std;

namespace cube
{
class UnsignedValue;
extern Preallocator<UnsignedValue> uint32_preallocator;

/**
 * Value works with 4 bytes of the unsigned int as teh whole int value, and as 4 bytes (endianess). Therefore it
 * defined a union.
 */
typedef union
{
    uint32_t uValue;
    char     aValue[ sizeof( uint32_t ) ];
} u_value_t;



class UnsignedValue : public Value
{
protected:
    u_value_t value;                     /// "heart" of the value.
public:
    UnsignedValue();
//     UnsignedValue( uint8_t );
//     UnsignedValue( int8_t );
//     UnsignedValue( uint16_t );
//     UnsignedValue( int16_t );
    UnsignedValue( uint32_t );
//     UnsignedValue( int32_t );
//     UnsignedValue( uint64_t );
//     UnsignedValue( int64_t );
//
//     UnsignedValue( double );
//     UnsignedValue( char* );

    virtual
    ~UnsignedValue()
    {
    };
    inline
    virtual unsigned
    getSize()
    {
        return sizeof( uint32_t );
    }

    inline
    virtual double
    getDouble()
    {
        return ( double )value.uValue;
    };

    virtual uint16_t
    getUnsignedShort();
    virtual int16_t
    getSignedShort();
    virtual uint32_t
    getUnsignedInt();
    virtual int32_t
    getSignedInt();
    virtual uint64_t
    getUnsignedLong();
    virtual int64_t
    getSignedLong();
    virtual char
    getChar();
    virtual string
    getString();

    virtual char*
    fromStream( char* );
    virtual char*
    toStream( char* );

    inline
    virtual Value*
    clone()
    {
        return new UnsignedValue( 0 );
    }
    inline
    virtual Value*
    copy()
    {
        return new UnsignedValue( value.uValue );
    }
/*
    virtual UnsignedValue
    operator+( const UnsignedValue& );
    virtual UnsignedValue
    operator-( const UnsignedValue& );*/

    inline
    virtual void
    operator+=( Value* chval )
    {
        if ( chval == NULL )
        {
            return;
        }
        value.uValue += ( ( UnsignedValue* )chval )->value.uValue;
    }

    inline
    virtual void
    operator-=( Value* chval )
    {
        if ( chval == NULL )
        {
            return;
        }
        value.uValue -= ( ( UnsignedValue* )chval )->value.uValue;
    }

    inline
    virtual void
    operator*=( double dval )
    {
        value.uValue *= dval;
    }                                 // needed by algebra tools

    inline
    virtual void
    operator/=( double dval )
    {
        if ( dval == 0. )
        {
            cerr << "ERROR: DEVISION BY ZERO!" << endl;
        }
        value.uValue /= dval;
    }


    void*
    operator new( size_t size );
    void
    operator delete( void* p );

    virtual void
    Free()
    {
        delete ( UnsignedValue* )this;
    }

    virtual void
    operator=( double );

//     virtual void
//     operator=( char );
//     virtual void
//     operator=( uint16_t );
//     virtual void
//     operator=( uint32_t );
//     virtual void
//     operator=( uint64_t );
//     virtual void
//     operator=( int16_t );
//     virtual void
//     operator=( int32_t );
//     virtual void
//     operator=( int64_t );
    virtual void
    operator=( Value* );

//     virtual void
//     operator=( string );
//
/*    virtual UnsignedValue
    operator=( UnsignedValue );  */                             /// Assignemnt operator.
//
    virtual bool
    isZero()
    {
        return value.uValue == 0;
    };

    virtual DataType
    myDataType()
    {
        return CUBE_DATA_TYPE_UINT32;
    };

    virtual void
    normalizeWithClusterCount( uint64_t );
};
}
#endif
